@@ -1,13 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007,2008 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007 -- leonerd@leonerd.org.uk
 
 package IO::Async::SignalProxy;
 
 use strict;
 
-our $VERSION = '0.16.001';
+our $VERSION = '0.01';
 
 use base qw( IO::Async::Notifier );
 
@@ -16,24 +16,46 @@ use Carp;
 use POSIX qw( EAGAIN SIG_BLOCK SIG_SETMASK sigprocmask );
 use IO::Handle;
 
-use warnings qw();
-
 =head1 NAME
 
-C<IO::Async::SignalProxy> - handle POSIX signals with C<IO::Async>-based IO
+C<IO::Async::SignalProxy> - a class to allow handling of POSIX signals with
+C<IO::Async>-based IO
 
 =head1 SYNOPSIS
 
-This object class is for internal use by C<IO::Async::Loop> subclasses. It is
-not intended for use externally.
+ use IO::Async::SignalProxy;
+
+ my $sigproxy = IO::Async::SignalProxy->new(
+    signal_HUP => sub { reread_config() },
+ );
+
+ my $set = IO::Async::Set::...
+ $set->add( $sigproxy );
 
 =head1 DESCRIPTION
 
 This module provides a class that allows POSIX signals to be handled safely
-alongside other IO operations on filehandles in an C<IO::Async::Loop>. Because
+alongside other IO operations on filehandles in an C<IO::Async::Set>. Because
 signals could arrive at any time, care must be taken that they do not
 interrupt the normal flow of the program, and are handled at the same time as
-other events in the C<IO::Async::Loop>'s results.
+other events in the C<IO::Async::Set>'s results.
+
+=cut
+
+=head1 CONSTRUCTOR
+
+=cut
+
+=head2 $proxy = IO::Async::SignalProxy->new( %params )
+
+This function returns a new instance of a C<IO::Async::SignalProxy> object.
+The C<%params> hash takes keys that specify callback functions to run when
+signals arrive. They are all of the form
+
+ signal_$NAME => sub { ... }
+
+where C<$NAME> is the basic POSIX name for the signal, such as C<TERM> or
+C<CHLD>.
 
 =cut
 
@@ -42,9 +64,7 @@ sub new
    my $class = shift;
    my ( %params ) = @_;
 
-   my $loop = delete $params{loop} or croak "Expected a 'loop'";
-
-   my ( $reader, $writer ) = $loop->pipepair() or croak "Cannot pipe() - $!";
+   pipe( my ( $reader, $writer ) ) or croak "Cannot pipe() - $!";
 
    $reader->blocking( 0 );
    $writer->blocking( 0 );
@@ -54,18 +74,46 @@ sub new
       read_handle => $reader,
    );
 
-   $self->{pipe_writing_end} = $writer;
-
    $self->{restore_SIG}  = {}; # {$signame} = value
    $self->{callbacks} = {}; # {$signame} = CODE
 
    # This variable is race-sensitive - read the notes in __END__ section
    # before attempting to modify this code.
-   $self->{signal_queue} = [];
+   my $signal_queue = $self->{signal_queue} = [];
 
-   $self->{sigset_block} = POSIX::SigSet->new();
+   my $sigset_block = POSIX::SigSet->new();
 
-   $loop->add( $self );
+   # Find all the signal handler callbacks
+   foreach my $signame ( map { m/^signal_(.*)$/ ? $1 : () } keys %params ) {
+      $self->{callbacks}->{$signame} = $params{"signal_$signame"};
+
+      exists $SIG{$signame} or croak "Unrecognised signal name $signame";
+
+      # Don't allow anyone to trash an existing signal handler
+      !defined $SIG{$signame} or !ref $SIG{$signame} or croak "Cannot override signal handler for $signame";
+
+      $self->{restore_SIG}->{$signame} = $SIG{$signame};
+
+      $SIG{$signame} = sub {
+         # This signal handler is race-sensitive - read the notes in the
+         # __END__ section before attempting to modify this code.
+         if( !@$signal_queue ) {
+            syswrite( $writer, "\0" );
+         }
+         push @$signal_queue, $signame;
+      };
+
+      my $signum;
+      {
+         no strict 'refs';
+         local @_;
+         $signum = &{"POSIX::SIG$signame"};
+      }
+
+      $sigset_block->addset( $signum );
+   }
+
+   $self->{sigset_block} = $sigset_block;
 
    return $self;
 }
@@ -76,11 +124,14 @@ sub DESTROY
 
    my $restore_SIG = $self->{restore_SIG};
 
-   $self->detach( $_ ) foreach keys %$restore_SIG;
+   # When we saved the original value, we might have got an undef. But %SIG
+   # doesn't like having undef assigned back in, so we need to translate
+
+   $SIG{$_} = $restore_SIG->{$_} || 'DEFAULT' foreach keys %$restore_SIG;
 }
 
 # protected
-sub on_read_ready
+sub read_ready
 {
    my $self = shift;
 
@@ -128,92 +179,10 @@ sub on_read_ready
 
    foreach( @caught_signals ) {
       my $callback = $self->{callbacks}->{$_};
-      # This shouldn't fail, but I've seen what looked like a race condition
-      # failure on a Solaris 8 + perl 5.6.1 machine here. This message should
-      # at least help find the problem
-      defined $callback or croak "Undefined callback for signal $_";
-      ref $callback eq "CODE" or croak "Callback for signal $_ is not a CODE ref";
-
       $callback->();
    }
 }
 
-sub attach
-{
-   my $self = shift;
-   my ( $signal, $code ) = @_;
-
-   if( $signal eq "CHLD" ) {
-      # We make special exception to allow the ChildManager to do this
-      caller(1) eq "IO::Async::ChildManager" or
-         carp "Attaching to SIGCHLD is not advised - use the IO::Async::ChildManager instead";
-   }
-
-   exists $SIG{$signal} or croak "Unrecognised signal name $signal";
-
-   # Don't allow anyone to trash an existing signal handler
-   !defined $SIG{$signal} or !ref $SIG{$signal} or croak "Cannot override signal handler for $signal";
-
-   ref $code eq "CODE" or croak 'Expected $code as a CODE reference';
-
-   $self->{callbacks}->{$signal} = $code;
-
-   $self->{restore_SIG}->{$signal} = $SIG{$signal};
-
-   my $writer = $self->{pipe_writing_end};
-   my $signal_queue = $self->{signal_queue};
-
-   # This closure cannot refer to $self because that would keep the reference
-   # count on the object and prevent it from being DESTROYed when it goes out
-   # of scope from the caller
-   $SIG{$signal} = sub {
-      # This signal handler is race-sensitive - read the notes in the
-      # __END__ section before attempting to modify this code.
-
-      # Protect the interrupted code against unexpected modifications of $!,
-      # such as the line just after the poll() call in IO::Async::Loop::IO_Poll
-      local $!;
-
-      if( !@$signal_queue ) {
-         syswrite( $writer, "\0" );
-      }
-      push @$signal_queue, $signal;
-   };
-
-   my $signum;
-   {
-      no strict 'refs';
-      local @_;
-      $signum = &{"POSIX::SIG$signal"};
-   }
-
-   my $sigset_block = $self->{sigset_block};
-   $sigset_block->addset( $signum );
-}
-
-sub detach
-{
-   my $self = shift;
-   my ( $signal ) = @_;
-
-   my $restore_SIG = $self->{restore_SIG};
-
-   exists $restore_SIG->{$signal} or croak "Signal name $signal is not currently attached";
-
-   # When we saved the original value, we might have got an undef. But %SIG
-   # doesn't like having undef assigned back in, so we need to translate
-   $SIG{$signal} = $restore_SIG->{$signal} || 'DEFAULT';
-
-   delete $restore_SIG->{$signal};
-   delete $self->{callbacks}->{$signal};
-}
-
-sub signals
-{
-   my $self = shift;
-   return keys %{ $self->{callbacks} };
-}
-
 # Keep perl happy; keep Britain tidy
 1;
 
@@ -223,11 +192,11 @@ Some notes on implementation
 ============================
 
 The purpose of this object class is to safely call the required signal-
-handling callback code as part of the usual IO::Async::Loop loop. This is done 
+handling callback code as part of the usual IO::Async::Set loop. This is done 
 by keeping a queue of incoming signal names in the array referenced by 
 $self->{signal_queue}. The object installs its own signal handler which pushes
-the signal name to this array, and the on_read_ready method then replays them
-out again.
+the signal name to this array, and the read_ready method then replays them out
+again.
 
 In order to safely interact with any sort of file-based asynchronous IO (such
 as a select() or poll() system call), the object keeps both ends of a pipe.
@@ -238,7 +207,7 @@ emptyness of the signal queue array is maintained identically to the emptyness
 of the pipe. The value of the bytes in the pipe does not matter; only their
 presence is important.
 
-The on_read_ready method uses POSIX::sigprocmask() to mask off all the signals
+The read_ready method uses POSIX::sigprocmask() to mask off all the signals
 that the object is handling, so that it can atomically read the pipe and empty
 the signal queue array, without a danger of a race condition if another signal
 arrives while it is doing this. The only race condition that remains is the
@@ -251,16 +220,26 @@ the critical stage:
       }
 
 In this case, no bad effects will happen. There will simply be two bytes
-written into the pipe, rather than the usual one. The on_read_ready handler
-will attempt a sysread() of up to 8192 bytes, and there are usually no more
-than a handful of different signal handlers registered, so this will usually
-not be a problem. In the exceedingly-unlikely event that more than 8192
-different user-defined signal handlers have in fact been registered, and every
-one of them is fired at the same time, and every one races with all the others
-in the way indicated above, then all that will happen is that the pipe will
-remain non-empty while the signal queue array is empty. In this case, the
-on_read_ready handler will be fired again, read up-to 8192 more bytes, then
-find the queue to be empty, and return again.
+written into the pipe, rather than the usual one. The read_ready handler will
+attempt a sysread() of up to 8192 bytes, and there are usually no more than a 
+handful of different signal handlers registered, so this will usually not be a
+problem. In the exceedingly-unlikely event that more than 8192 different user-
+defined signal handlers have in fact been registered, and every one of them is
+fired at the same time, and every one races with all the others in the way
+indicated above, then all that will happen is that the pipe will remain non-
+empty while the signal queue array is empty. In this case, the read_ready
+handler will be fired again, read up-to 8192 more bytes, then find the queue
+to be empty, and return again.
+
+=head1 SEE ALSO
+
+=over 4
+
+=item *
+
+L<POSIX> for the C<SIGI<name>> constants
+
+=back
 
 =head1 AUTHOR
 
